@@ -125,10 +125,59 @@ class TempfilePatcher:
             tempfile._TemporaryFileCloser.close = self.tempfile_cleanup  # type: ignore[module-attr]
         else:
             tempfile._TemporaryFileCloser.cleanup = self.tempfile_cleanup  # type: ignore[module-attr]
+        self.tempfile_cleanup = None
         # reset the cached tempdir in tempfile
         tempfile.tempdir = None
 
 
+class LineCachePatcher:
+    """Handles linecache patching for newer Python versions."""
+
+    def __init__(self):
+        self.linecache_updatecache = None
+        self.linecache_checkcache = None
+
+    def start_patching(self):
+        if self.linecache_updatecache is not None:
+            return
+
+        def checkcache(filename=None):
+            """Calls the original linecache.checkcache making sure no fake OS calls
+            are used."""
+            with use_original_os():
+                return self.linecache_checkcache(filename)
+
+        def updatecache(filename, module_globals=None):
+            """Calls the original linecache.updatecache making sure no fake OS calls
+            are used."""
+
+            with use_original_os():
+                # workaround for updatecache problem with pytest under Windows, see #1096
+                if not filename.endswith(r"pytest.exe\__main__.py"):
+                    return self.linecache_updatecache(filename, module_globals)
+                return []
+
+        if (
+            sys.version_info >= (3, 13)
+            or sys.platform == "win32"
+            and sys.version_info >= (3, 12)
+        ):
+            # in linecache, 'os' is now imported locally, which involves the
+            # dynamic patcher, therefore we patch the affected functions
+            self.linecache_updatecache = linecache.updatecache
+            linecache.updatecache = updatecache
+            self.linecache_checkcache = linecache.checkcache
+            linecache.checkcache = checkcache
+
+    def stop_patching(self):
+        if self.linecache_updatecache is None:
+            return
+        linecache.updatecache = self.linecache_updatecache
+        linecache.checkcache = self.linecache_checkcache
+        self.linecache_updatecache = None
+        self.linecache_checkcache = None
+
+
 def patchfs(
     _func: Optional[Callable] = None,
     *,
@@ -529,7 +578,7 @@ class Patcher:
     FS_DEFARGS: List[Tuple[FunctionType, int, Callable[..., Any]]] = []
     SKIPPED_FS_MODULES: Dict[str, Set[Tuple[ModuleType, str]]] = {}
 
-    assert None in SKIPMODULES, "sys.modules contains 'None' values;" " must skip them."
+    assert None in SKIPMODULES, "sys.modules contains 'None' values; must skip them."
 
     IS_WINDOWS = sys.platform in ("win32", "cygwin")
 
@@ -615,9 +664,8 @@ class Patcher:
         # save the original open function for use in pytest plugin
         self.original_open = open
         self.patch_open_code = patch_open_code
-        self.linecache_updatecache = None
-        self.linecache_checkcache = None
         self.tempfile_patcher = TempfilePatcher()
+        self.linecache_patcher = LineCachePatcher()
 
         if additional_skip_names is not None:
             skip_names = [
@@ -688,18 +736,8 @@ class Patcher:
         self._dyn_patcher: Optional[DynamicPatcher] = None
         self._patching = False
         self._paused = False
-
-    def checkcache(self, filename=None):
-        """Calls the original linecache.checkcache making sure no fake OS calls
-        are used."""
-        with use_original_os():
-            return self.linecache_checkcache(filename)
-
-    def updatecache(self, filename, module_globals=None):
-        """Calls the original linecache.updatecache making sure no fake OS calls
-        are used."""
-        with use_original_os():
-            return self.linecache_updatecache(filename, module_globals)
+        self.has_copy_file_range = False
+        self.has_copy_file = False
 
     @classmethod
     def clear_fs_cache(cls) -> None:
@@ -986,9 +1024,22 @@ class Patcher:
         if self.has_fcopy_file:
             shutil._HAS_FCOPYFILE = False  # type: ignore[attr-defined]
 
+        self.has_copy_file_range = (
+            sys.platform == "linux"
+            and hasattr(shutil, "_USE_CP_COPY_FILE_RANGE")
+            and shutil._USE_CP_COPY_FILE_RANGE
+        )
+        if self.has_copy_file_range:
+            shutil._USE_CP_COPY_FILE_RANGE = False  # type: ignore[attr-defined]
+
         # do not use the fd functions, as they may not be available in the target OS
         if hasattr(shutil, "_use_fd_functions"):
             shutil._use_fd_functions = False  # type: ignore[module-attr]
+        # in Python 3.14, _rmtree_impl is set at load time based on _use_fd_functions
+        # the safe version cannot be used at the moment as it used asserts of type
+        # 'assert func is os.rmtree', which do not work with the fake versions
+        if hasattr(shutil, "_rmtree_impl"):
+            shutil._rmtree_impl = shutil._rmtree_unsafe  # type: ignore[attr-defined]
 
         with warnings.catch_warnings():
             # ignore warnings, see #542 and #614
@@ -1009,14 +1060,7 @@ class Patcher:
             self._patching = True
             self._paused = False
 
-            if sys.version_info >= (3, 13):
-                # in linecache, 'os' is now imported locally, which involves the
-                # dynamic patcher, therefore we patch the affected functions
-                self.linecache_updatecache = linecache.updatecache
-                linecache.updatecache = self.updatecache
-                self.linecache_checkcache = linecache.checkcache
-                linecache.checkcache = self.checkcache
-
+            self.linecache_patcher.start_patching()
             self.tempfile_patcher.start_patching()
 
             self.patch_modules()
@@ -1038,7 +1082,8 @@ class Patcher:
         if sys.version_info >= (3, 13):
             globber = glob._StringGlobber  # type: ignore[module-attr]
             globber.lstat = staticmethod(os.lstat)
-            globber.scandir = staticmethod(os.scandir)
+            if sys.version_info < (3, 14):
+                globber.scandir = staticmethod(os.scandir)
 
     def patch_functions(self) -> None:
         assert self._stubs is not None
@@ -1115,6 +1160,8 @@ class Patcher:
         self.stop_patching()
         if self.has_fcopy_file:
             shutil._HAS_FCOPYFILE = True  # type: ignore[attr-defined]
+        if self.has_copy_file_range:
+            shutil._USE_CP_COPY_FILE_RANGE = True  # type: ignore[attr-defined]
 
         reset_ids()
         if self.is_doc_test:
@@ -1134,9 +1181,7 @@ class Patcher:
                 self._dyn_patcher.cleanup()
                 sys.meta_path.pop(0)
             self.tempfile_patcher.stop_patching()
-            if self.linecache_updatecache is not None:
-                linecache.updatecache = self.linecache_updatecache
-                linecache.checkcache = self.linecache_checkcache
+            self.linecache_patcher.stop_patching()
             self._set_glob_os_functions()
 
     @property
